/*
Copyright 2023.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package v1beta1

import (
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
)

// AnalysisValueTemplateSpec defines the desired state of AnalysisValueTemplate
type AnalysisValueTemplateSpec struct {
	// Provider refers to the KeptnMetricsProvider which should be used to retrieve the data
	Provider ObjectReference `json:"provider" yaml:"provider"`
	// Query represents the query to be run. It can include placeholders that are defined using the go template
	// syntax. More info on go templating - https://pkg.go.dev/text/template
	Query string `json:"query" yaml:"query"`
}

//+kubebuilder:object:root=true
//+kubebuilder:subresource:status
//+kubebuilder:printcolumn:name="Provider",type=string,JSONPath=`.spec.provider.name`

// AnalysisValueTemplate is the Schema for the analysisvaluetemplates API
type AnalysisValueTemplate struct {
	metav1.TypeMeta `json:",inline" yaml:",inline"`
	// +optional
	metav1.ObjectMeta `json:"metadata,omitempty" yaml:"metadata,omitempty"`

	// Spec contains the specification for the AnalysisValueTemplate
	// +optional
	Spec AnalysisValueTemplateSpec `json:"spec,omitempty" yaml:"spec,omitempty"`
	// unused field
	// +optional
	Status string `json:"status,omitempty" yaml:"status,omitempty"`
}

//+kubebuilder:object:root=true

// AnalysisValueTemplateList contains a list of AnalysisValueTemplate resources
type AnalysisValueTemplateList struct {
	metav1.TypeMeta `json:",inline"`
	// +optional
	metav1.ListMeta `json:"metadata,omitempty"`
	Items           []AnalysisValueTemplate `json:"items"`
}

func init() {
	SchemeBuilder.Register(&AnalysisValueTemplate{}, &AnalysisValueTemplateList{})
}
